from typing import Generic, Iterable, Literal, Optional, TypeVar  # noqa: D100

import pydantic
from karp.lex_core.value_objects import UniqueId, make_unique_id
from pydantic.generics import GenericModel

from .base import Command

T = TypeVar("T")


class GenericAddEntry(GenericModel, Generic[T], Command):  # noqa: D101
    id: UniqueId = pydantic.Field(default_factory=make_unique_id)  # noqa: A003
    resource_id: str
    entry: T
    message: str
    cmdtype: Literal["add_entry"] = "add_entry"


class AddEntry(GenericAddEntry[dict]):
    """Command to add an entry."""


class AddEntries(Command):  # noqa: D101
    resource_id: str
    entries: Iterable[dict]
    message: str
    chunk_size: int = 0


class AddEntriesInChunks(AddEntries):  # noqa: D101
    chunk_size: int


class DeleteEntry(Command):  # noqa: D101
    resource_id: str
    id: UniqueId  # noqa: A003
    version: int
    message: Optional[str] = None
    cmdtype: Literal["delete_entry"] = "delete_entry"


class ImportEntries(AddEntries):  # noqa: D101
    pass


class ImportEntriesInChunks(AddEntriesInChunks):  # noqa: D101
    pass


class GenericUpdateEntry(GenericModel, Generic[T], Command):  # noqa: D101
    resource_id: str
    id: UniqueId  # noqa: A003
    version: int
    entry: T
    message: str
    force: bool = False
    cmdtype: Literal["update_entry"] = "update_entry"


class UpdateEntry(GenericUpdateEntry[dict]):  # noqa: D101
    ...



import pydantic
from karp.lex_core.commands import (
    AddEntry,
    CreateResource,
    GenericAddEntry,
    GenericCreateResource,
    GenericUpdateEntry,
    GenericUpdateResource,
    LexCommand,
    UpdateEntry,
    UpdateResource,
)


class Ex(pydantic.BaseModel):
    baseform: str


class TestAddEntry:
    def test_generic_cmd_exported_is_imported_as_nongeneric(self):  # noqa: ANN201
        cmd = GenericAddEntry[Ex](
            entry={"baseform": "ord"},
            message="add",
            user="user1",
            resourceId="abc",
        )
        cmd_dict = cmd.serialize()

        assert isinstance(LexCommand(command=cmd_dict).command, AddEntry)


class TestUpdateEntry:
    def test_generic_cmd_exported_is_imported_as_nongeneric(self):  # noqa: ANN201
        cmd = GenericUpdateEntry[Ex](
            id="01GSAHD0K063FBMFE19BFDM4E9",
            entry={"baseform": "ord"},
            version=3,
            message="add",
            user="user1",
            resourceId="abc",
        )
        cmd_dict = cmd.serialize()

        assert isinstance(LexCommand(command=cmd_dict).command, UpdateEntry)


class ExResource(pydantic.BaseModel):
    fields: dict


class TestCreateResource:
    def test_generic_cmd_exported_is_imported_as_nongeneric(self):  # noqa: ANN201
        cmd = GenericCreateResource[ExResource](
            config={"fields": {"baseform": {"type": "string"}}},
            entryRepoId="01GSAHD0K063FBMFE19BFDM4E9",
            message="add",
            user="user1",
            resourceId="abc",
            name="ABC",
        )
        cmd_dict = cmd.serialize()

        assert isinstance(LexCommand(command=cmd_dict).command, CreateResource)


class TestUpdateResource:
    def test_generic_cmd_exported_is_imported_as_nongeneric(self):  # noqa: ANN201
        cmd = GenericUpdateResource[ExResource](
            id="01GSAHD0K063FBMFE19BFDM4E9",
            config={"fields": {"baseform": {"type": "string"}}},
            version=3,
            message="add",
            user="user1",
            name="abc",
        )
        cmd_dict = cmd.serialize()

        assert isinstance(LexCommand(command=cmd_dict).command, UpdateResource)



from typing import Type

import pytest
from karp.lex_core.commands import (
    AddEntry,
    CreateEntryRepository,
    CreateResource,
    DeleteEntry,
    DeleteEntryRepository,
    DeleteResource,
    EntryCommand,
    EntryRepoCommand,
    LexCommand,
    PublishResource,
    ResourceCommand,
    SetEntryRepoId,
    UpdateEntry,
    UpdateResource,
)


@pytest.mark.parametrize(
    "data, expected_cls",
    [
        (
            {
                "resourceId": "abc",
                "cmdtype": "add_entry",
                "entry": {},
                "user": "user1",
                "message": "add",
            },
            AddEntry,
        ),
        (
            {
                "id": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                "resourceId": "abc",
                "cmdtype": "delete_entry",
                "user": "user1",
                "message": "add",
            },
            DeleteEntry,
        ),
        (
            {
                "id": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                "resourceId": "abc",
                "cmdtype": "update_entry",
                "entry": {},
                "user": "user1",
                "message": "add",
            },
            UpdateEntry,
        ),
    ],
)
def test_entry_command(data: dict, expected_cls: Type) -> None:
    lex_cmd = LexCommand(command=data)

    assert isinstance(lex_cmd.command, expected_cls)
    assert isinstance(EntryCommand(command=data).command, expected_cls)


@pytest.mark.parametrize(
    "data, expected_cls",
    [
        (
            {
                "name": "abc",
                "repositoryType": "default",
                "cmdtype": "create_entry_repository",
                "config": {},
                "user": "user1",
                "message": "add",
            },
            CreateEntryRepository,
        ),
        (
            {
                "id": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                "cmdtype": "delete_entry_repository",
                "user": "user1",
                "message": "add",
            },
            DeleteEntryRepository,
        ),
    ],
)
def test_entry_repo_command(data: dict, expected_cls: Type) -> None:
    lex_cmd = LexCommand(command=data)

    assert isinstance(lex_cmd.command, expected_cls)
    assert isinstance(EntryRepoCommand(command=data).command, expected_cls)


@pytest.mark.parametrize(
    "data, expected_cls",
    [
        (
            {
                "resourceId": "abc",
                "cmdtype": "create_resource",
                "config": {},
                "entryRepoId": "01GSAHD0K063FBMFE19BFDM4E9",
                "name": "ABC",
                "user": "user1",
                "message": "add",
            },
            CreateResource,
        ),
        (
            {
                "id": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                # "resourceId": "abc",
                "cmdtype": "delete_resource",
                "user": "user1",
                "message": "add",
            },
            DeleteResource,
        ),
        (
            {
                # "id": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                "resourceId": "abc",
                "name": "abc",
                "cmdtype": "update_resource",
                "config": {},
                "user": "user1",
                "message": "add",
            },
            UpdateResource,
        ),
        (
            {
                "id": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                # "resourceId": "abc",
                "cmdtype": "publish_resource",
                "user": "user1",
                "message": "add",
            },
            PublishResource,
        ),
        (
            {
                "entryRepoId": "01GSAHD0K063FBMFE19BFDM4E9",
                "version": 4,
                "resourceId": "abc",
                "cmdtype": "set_entry_repo_id",
                "user": "user1",
                "message": "add",
            },
            SetEntryRepoId,
        ),
    ],
)
def test_resource_command(data: dict, expected_cls: Type) -> None:
    lex_cmd = LexCommand(command=data)

    assert isinstance(lex_cmd.command, expected_cls)
    assert isinstance(ResourceCommand(command=data).command, expected_cls)



from datetime import datetime

import pytest
from karp.lex_core.value_objects import UniqueId, make_unique_id


def test_unique_ids_are_sortable():  # noqa: ANN201
    assert make_unique_id() > make_unique_id(datetime(1999, 12, 31))


class TestUniqueId:
    def test_bad_tyoe_raises_type_error(self) -> None:
        with pytest.raises(TypeError):
            UniqueId.validate(None)

    def test_bad_input_raises_value_error(self) -> None:
        with pytest.raises(ValueError):
            UniqueId.validate("not-an-ulid")



from karp.lex_core.commands import CreateEntryRepository


class TestCreateEntryRepository:
    def test_from_dict_works(self) -> None:
        cmd = CreateEntryRepository.from_dict(
            {
                "resource_id": "abc",
                "resource_name": "Abc",
            },
            # entry_repo_id="01GSAHD0K063FBMFE19BFDM4E9",
            user="user1",
        )
        assert cmd.cmdtype == "create_entry_repository"



from typing import Literal, Optional  # noqa: D100

import pydantic
from karp.lex_core.value_objects import UniqueId, make_unique_id

from .base import Command


class CreateEntryRepository(Command):  # noqa: D101
    id: UniqueId = pydantic.Field(default_factory=make_unique_id)  # noqa: A003
    repository_type: str
    name: str
    connection_str: Optional[str] = None
    config: dict
    message: str
    user: str
    cmdtype: Literal["create_entry_repository"] = "create_entry_repository"

    @classmethod
    def from_dict(  # noqa: D102
        cls, data: dict, *, user: str, message: Optional[str] = None
    ) -> "CreateEntryRepository":
        return cls(
            # id=make_unique_id_str(),
            repositoryType=data.pop("repository_type", "default"),
            name=data.pop("resource_id"),
            connectionStr=data.pop("connection_str", None),
            config=data,
            user=user,
            message=message or "Entry repository created",
        )


class DeleteEntryRepository(Command):  # noqa: D101
    id: UniqueId  # noqa: A003
    version: int
    message: str
    user: str
    cmdtype: Literal["delete_entry_repository"] = "delete_entry_repository"



"""Base class for lex commands"""  # noqa: D400, D415

from datetime import datetime, timezone
from typing import Optional

import pydantic
from karp.lex_core import alias_generators


def utc_now() -> float:
    """A UTC timestamp in seconds.

    This function may return the same timestamp twice.
    """  # noqa: D401
    return datetime.now(timezone.utc).timestamp()


class Command(pydantic.BaseModel):  # noqa: D101
    timestamp: float = pydantic.Field(default_factory=utc_now)
    user: str
    message: Optional[str]

    class Config:  # noqa: D106
        arbitrary_types_allowed = True
        extra = "forbid"
        alias_generator = alias_generators.to_lower_camel

    def serialize(self) -> dict:
        """Export as dict with alias and without None:s."""  # noqa: D202

        return self.dict(by_alias=True, exclude_none=True)



"""Alias generators."""


def to_lower_camel(s: str) -> str:
    """Transform snake_case to lowerCamelCase."""  # noqa: D202

    return "".join(
        word.capitalize() if i > 0 else word for i, word in enumerate(s.split("_"))
    )



from karp.lex_core.value_objects.unique_id import (  # noqa: D104
    UniqueId,
    UniqueIdStr,
    UniqueIdType,
    make_unique_id,
    make_unique_id_str,
)

__all__ = [
    "UniqueId",
    "UniqueIdStr",
    "UniqueIdType",
    "make_unique_id",
    "make_unique_id_str",
]



import pytest
from karp.lex_core.commands.resource_commands import (
    CreateResource,
    EntityOrResourceIdMixin,
)


class TestEntityOrResourceIdMixin:
    def test_given_both_raises_value_error(self):  # noqa: ANN201
        with pytest.raises(ValueError):
            EntityOrResourceIdMixin(
                resourceId="abc",
                id="01GSAHD0K063FBMFE19BFDM4E9",
            )

    def test_given_either_raises_value_error(self):  # noqa: ANN201
        with pytest.raises(ValueError):
            EntityOrResourceIdMixin()

    def test_both_given_none_raises_value_error(self):  # noqa: ANN201
        with pytest.raises(ValueError):
            EntityOrResourceIdMixin(
                resourceId=None,
                entityId=None,
            )


class TestCreateResource:
    def test_from_dict_works(self) -> None:
        cmd = CreateResource.from_dict(
            {
                "resource_id": "abc",
                "resource_name": "Abc",
            },
            entry_repo_id="01GSAHD0K063FBMFE19BFDM4E9",
        )
        assert cmd.cmdtype == "create_resource"



from karp.lex_core.dtos import EntryDto


def test_can_create_entry_dto():  # noqa: ANN201
    entry_dto = EntryDto(entry={"field": "value"})

    assert entry_dto.last_modified is None
    assert entry_dto.last_modified_by is None


def test_can_create_entry_dto_with_last_modified_by():  # noqa: ANN201
    entry_dto = EntryDto(
        entry={"field": "value"}, lastModifiedBy="username@example.com"
    )

    assert entry_dto.last_modified is None
    assert entry_dto.last_modified_by == "username@example.com"


def test_example_snake_case():  # noqa: ANN201
    data = {
        "id": "01GMMWP3ECHP7JEB9NVGRTZ7M6",
        "resource": "fulaord",
        "version": 1,
        "entry": {
            "baseform": "1- on -1",
            "id": "1-_on_-1..1",
            "wordforms": ["one to one", "one 2 one"],
            "text": "<p>även <em>one to one, one 2 one</em> (sl) flitigt använda i ...</p>",
        },
        "last_modified": 1671443451.340828,
        "last_modified_by": "local admin",
    }
    entry_dto = EntryDto(**data)

    assert entry_dto.id == data["id"]
    assert entry_dto.resource == data["resource"]
    assert entry_dto.version == data["version"]
    assert entry_dto.last_modified.timestamp() == data["last_modified"]  # type: ignore
    assert entry_dto.last_modified_by == data["last_modified_by"]

    serialized_entry = entry_dto.serialize()

    assert serialized_entry["id"] == data["id"]



"""Commands for lex resources."""

from typing import Generic, Literal, Optional, TypeVar

import pydantic
from karp.lex_core.value_objects.unique_id import (
    UniqueId,
    UniqueIdPrimitive,
    make_unique_id,
)
from pydantic.generics import GenericModel

from .base import Command

T = TypeVar("T")


class EntityOrResourceIdMixin(Command):  # noqa: D101
    resource_id: Optional[str]
    id: Optional[UniqueId]  # noqa: A003

    @pydantic.root_validator(pre=True)
    def resource_id_or_id(cls, values) -> dict:  # noqa: D102, ANN001
        resource_id = values["resourceId"] if "resourceId" in values else None
        if "id" in values and resource_id:
            raise ValueError("Can't give both 'id' and 'resourceId'")

        if resource_id:
            return dict(values) | {
                "id": None,
                "resourceId": resource_id,
            }
        elif "id" in values:
            return dict(values) | {
                "resourceId": None,
            }
        else:
            raise ValueError("Must give either 'id' or 'resourceId'")


class GenericCreateResource(GenericModel, Generic[T], Command):  # noqa: D101
    id: UniqueId = pydantic.Field(default_factory=make_unique_id)  # noqa: A003
    resource_id: str
    name: str
    config: T
    entry_repo_id: UniqueId
    cmdtype: Literal["create_resource"] = "create_resource"


class CreateResource(GenericCreateResource[dict]):
    """Command to create a resource."""

    @classmethod
    def from_dict(  # noqa: D102
        cls,
        data: dict,
        entry_repo_id: UniqueIdPrimitive,
        user: Optional[str] = None,
        message: Optional[str] = None,
    ) -> "CreateResource":
        try:
            resource_id = data.pop("resource_id")
        except KeyError as exc:
            raise ValueError("'resource_id' is missing") from exc
        try:
            name = data.pop("resource_name")
        except KeyError as exc:
            raise ValueError("'resource_name' is missing") from exc

        return cls(
            resourceId=resource_id,
            name=name,
            config=data,
            entryRepoId=entry_repo_id,
            user=user or "Unknown user",
            message=message or f"Resource '{resource_id}' created.",
        )


class GenericUpdateResource(EntityOrResourceIdMixin, GenericModel, Generic[T], Command):
    """Generic command for updating a resource."""

    version: int
    name: str
    config: T
    cmdtype: Literal["update_resource"] = "update_resource"


class UpdateResource(GenericUpdateResource[dict]):
    """Command for updating a resource."""


class PublishResource(EntityOrResourceIdMixin, Command):  # noqa: D101
    version: int
    cmdtype: Literal["publish_resource"] = "publish_resource"


class DeleteResource(EntityOrResourceIdMixin, Command):  # noqa: D101
    version: int
    cmdtype: Literal["delete_resource"] = "delete_resource"


class SetEntryRepoId(EntityOrResourceIdMixin, Command):  # noqa: D101
    entry_repo_id: UniqueId
    version: int
    cmdtype: Literal["set_entry_repo_id"] = "set_entry_repo_id"



from datetime import datetime  # noqa: D100
from typing import Generic, Optional, TypeVar

from karp.lex_core import alias_generators
from karp.lex_core.value_objects import UniqueIdStr
from pydantic import root_validator
from pydantic.generics import GenericModel

T = TypeVar("T")


class GenericEntryDto(GenericModel, Generic[T]):  # noqa: D101
    entry: T
    last_modified_by: Optional[str]
    last_modified: Optional[datetime]
    id: Optional[UniqueIdStr]  # noqa: A003
    message: Optional[str]
    version: Optional[int]
    resource: Optional[str]
    discarded: bool = False

    class Config:  # noqa: D106
        # extra = "forbid"
        alias_generator = alias_generators.to_lower_camel

    @root_validator(pre=True)
    @classmethod
    def allow_snake_case(cls, values):  # noqa: ANN206, D102, ANN001
        if "last_modified" in values:
            values["lastModified"] = values.pop("last_modified")
        if "last_modified_by" in values:
            values["lastModifiedBy"] = values.pop("last_modified_by")
        if "entity_id" in values:
            values["entityId"] = values.pop("entity_id")
        return values

    def serialize(self):  # noqa: ANN201, D102
        return self.dict(by_alias=True, exclude_none=True)


class EntryDto(GenericEntryDto[dict]):  # noqa: D101
    ...



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['lex_core',
 'lex_core.commands',
 'lex_core.dtos',
 'lex_core.tests',
 'lex_core.value_objects']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.4,<2.0.0', 'ulid-py>=1.1.0,<2.0.0']

setup_kwargs = {
    'name': 'karp-lex-core',
    'version': '0.5.1',
    'description': 'The core of karp-lex',
    'long_description': '# karp-lex-core\n\n- **karp-lex-core** [![PyPI version](https://badge.fury.io/py/karp-lex-core.svg)](https://badge.fury.io/py/karp-lex-core)\n\nThe core for karp-lex\n\nContains commands, entry_dto and unique_id for `karp.lex` in `karp.backend`.\n\n# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/).\n\n## Latest Changes\n\n## 0.4.0\n### Added\n* Added Generic Commands. PR [#224](https://github.com/spraakbanken/karp-backend/pull/224) by [@kod-kristoff](https://github.com/kod-kristoff).\n* Added cmdtype tags. PR [#223](https://github.com/spraakbanken/karp-backend/pull/223) by [@kod-kristoff](https://github.com/kod-kristoff).\n\n## 0.3.2\n\n* Repo move to https://github/spraakbanken/karp-backend from https://github/spraakbanken/karp-lex-core',
    'author': 'Språkbanken at the University of Gothenburg',
    'author_email': 'sb-info@svenska.gu.se',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://spraakbanken.gu.se',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""Commands top-level module."""

from typing import Union

from pydantic import BaseModel, Field

from .entry_commands import (
    AddEntries,
    AddEntriesInChunks,
    AddEntry,
    DeleteEntry,
    GenericAddEntry,
    GenericUpdateEntry,
    ImportEntries,
    ImportEntriesInChunks,
    UpdateEntry,
)
from .entry_repo_commands import CreateEntryRepository, DeleteEntryRepository
from .resource_commands import (
    CreateResource,
    DeleteResource,
    GenericCreateResource,
    GenericUpdateResource,
    PublishResource,
    SetEntryRepoId,
    UpdateResource,
)

__all__ = [
    # Entry commands
    "AddEntries",
    "AddEntriesInChunks",
    "AddEntry",
    "DeleteEntry",
    "GenericAddEntry",
    "GenericUpdateEntry",
    "ImportEntries",
    "ImportEntriesInChunks",
    "UpdateEntry",
    # EntryRepo commands
    "CreateEntryRepository",
    "DeleteEntryRepository",
    # Resource commands
    "CreateResource",
    "DeleteResource",
    "GenericCreateResource",
    "GenericUpdateResource",
    "PublishResource",
    "SetEntryRepoId",
    "UpdateResource",
]


EntryCommandType = Union[AddEntry, DeleteEntry, UpdateEntry]


EntryRepoCommandType = Union[CreateEntryRepository, DeleteEntryRepository]
ResourceCommandType = Union[
    CreateResource, DeleteResource, PublishResource, SetEntryRepoId, UpdateResource
]


class LexCommand(BaseModel):  # noqa: D101
    command: Union[EntryCommandType, EntryRepoCommandType, ResourceCommandType] = Field(
        ..., discriminator="cmdtype"
    )


class EntryCommand(BaseModel):  # noqa: D101
    command: EntryCommandType = Field(..., discriminator="cmdtype")


class EntryRepoCommand(BaseModel):  # noqa: D101
    command: EntryRepoCommandType = Field(..., discriminator="cmdtype")


class ResourceCommand(BaseModel):  # noqa: D101
    command: ResourceCommandType = Field(..., discriminator="cmdtype")



"""Handle of unique ids."""
import typing

import ulid
import ulid.codec

# UniqueId = ulid.ULID

UniqueIdPrimitive = ulid.api.api.ULIDPrimitive
# UniqueIdPrimitive = typing.Union[ulid.api.api.ULIDPrimitive, UniqueIdStr]


class UniqueId(ulid.ULID):  # noqa: D101
    @classmethod
    def __modify_schema__(cls, field_schema):  # noqa: ANN206, ANN001, D105
        field_schema.update(examples=["01BJQMF54D093DXEAWZ6JYRPAQ"])

    @classmethod
    def __get_validators__(cls):  # noqa: ANN206, D105
        yield cls.validate

    @classmethod
    def validate(cls, v) -> "UniqueId":  # noqa: D102, ANN001
        if isinstance(v, UniqueId):
            return v  # type: ignore
        if isinstance(v, ulid.ULID):
            return v  # type: ignore
        if not isinstance(v, UniqueIdPrimitive):  # type: ignore
            msg = f"Unsupported type ('{type(v)}')"
            raise TypeError(msg)
        try:
            return ulid.parse(v)  # type: ignore
        except ValueError as err:
            msg = "not a valid ULID"
            raise ValueError(msg) from err

    def __repr__(self) -> str:  # noqa: D105
        return f"UniqueId({super().__repr__()})"


UniqueIdType = (ulid.ULID, UniqueId)
typing_UniqueId = typing.Union[ulid.ULID, UniqueId]


def make_unique_id(
    t: typing.Optional[ulid.codec.TimestampPrimitive] = None,
) -> UniqueId:
    """Generate an UniqueId that are sortable.

    >>> from karp.lex_core.value_objects import make_unique_id
    >>> from datetime import datetime
    >>> old_id = make_unique_id(datetime(1999,12,31,23,59,59))
    >>> make_unique_id() > old_id
    True
    """
    return ulid.new() if t is None else ulid.from_timestamp(t)  # type: ignore


parse = ulid.parse


class UniqueIdStr(str):  # noqa: D101
    @classmethod
    def __modify_schema__(cls, field_schema):  # noqa: ANN206, ANN001, D105
        field_schema.update(examples=["01BJQMF54D093DXEAWZ6JYRPAQ"])

    @classmethod
    def __get_validators__(cls):  # noqa: ANN206, D105
        yield cls.validate

    @classmethod
    def validate(cls, v):  # noqa: ANN206, D102, ANN001
        if isinstance(v, (UniqueId, ulid.ULID)):
            return str(v)
        elif not isinstance(v, str):
            raise TypeError("string or UniqueId required")

        if len(v) != 26:
            raise ValueError("invalid uniqueid format")

        return cls(v)

    def __repr__(self) -> str:  # noqa: D105
        return f"UniqueIdStr({super().__repr__()})"


def make_unique_id_str(
    t: typing.Optional[ulid.codec.TimestampPrimitive] = None,
) -> UniqueIdStr:
    """Generate an UniqueIdStr that are sortable."""
    return UniqueIdStr(make_unique_id(t))



__version__ = "0.5.1"  # noqa: D104


from karp.lex_core import commands, value_objects
from karp.lex_core.dtos import EntryDto, GenericEntryDto

__all__ = ["EntryDto", "GenericEntryDto", "value_objects", "commands"]
